import threading
from typing import ClassVar, Optional

import httpx
from typing_extensions import Self

from . import types


class Client:
    _default: ClassVar[Optional["Client"]] = None
    _lock: ClassVar[threading.Lock] = threading.Lock()

    def __init__(
        self,
        *,
        settings: types.Settings,
    ) -> None:
        """
        Creates the AssemblyAI client.

        Args:
            settings: The settings to use for the client.
        """

        self._settings = settings.copy()

        if not self._settings.api_key:
            raise ValueError(
                "Please provide an API key via the ASSEMBLYAI_API_KEY environment variable or the global settings."
            )

        self._http_client = httpx.Client(
            base_url=self.settings.base_url,
            headers={
                "authorization": self.settings.api_key,
            },
            timeout=self.settings.http_timeout,
        )

    @property
    def settings(self) -> types.Settings:
        """
        Get the current settings.

        Returns:
            The current settings.
        """

        return self._settings

    @property
    def http_client(self) -> httpx.Client:
        """
        Get the current HTTP client.

        Returns:
            The current HTTP client.
        """

        return self._http_client

    @classmethod
    def get_default(cls) -> Self:
        """
        Return the default client.

        Returns:
            The default client with the default settings
        """
        from . import settings as default_settings

        if cls._default is None or cls._default.settings != default_settings:
            with cls._lock:
                if cls._default is None or cls._default.settings != default_settings:
                    cls._default = cls(settings=default_settings)

        return cls._default



import json
from typing import Any, Dict, Tuple

import factory
import httpx
import pytest
from pytest_httpx import HTTPXMock

import assemblyai as aai
from tests.unit import factories

aai.settings.api_key = "test"


class AutoChaptersResponseFactory(factories.TranscriptCompletedResponseFactory):
    chapters = factory.List([factory.SubFactory(factories.ChapterFactory)])


def __submit_mock_request(
    httpx_mock: HTTPXMock,
    mock_response: Dict[str, Any],
    config: aai.TranscriptionConfig,
) -> Tuple[Dict[str, Any], aai.Transcript]:
    """
    Helper function to abstract mock transcriber calls with given `TranscriptionConfig`,
    and perform some common assertions.
    """

    mock_transcript_id = mock_response.get("id", "mock_id")

    # Mock initial submission response (transcript is processing)
    mock_processing_response = factories.generate_dict_factory(
        factories.TranscriptProcessingResponseFactory
    )()

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json={
            **mock_processing_response,
            "id": mock_transcript_id,  # inject ID from main mock response
        },
    )

    # Mock polling-for-completeness response, with completed transcript
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_transcript_id}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_response,
    )

    # == Make API request via SDK ==
    transcript = aai.Transcriber().transcribe(
        data="https://example.org/audio.wav",
        config=config,
    )

    # Check that submission and polling requests were made
    assert len(httpx_mock.get_requests()) == 2

    # Extract body of initial submission request
    request = httpx_mock.get_requests()[0]
    request_body = json.loads(request.content.decode())

    return request_body, transcript


def test_auto_chapters_fails_without_punctuation(httpx_mock: HTTPXMock):
    """
    Tests whether the SDK raises an error before making a request
    if `auto_chapters` is enabled and `punctuation` is disabled
    """

    with pytest.raises(ValueError) as error:
        __submit_mock_request(
            httpx_mock,
            mock_response={},  # response doesn't matter, since it shouldn't occur
            config=aai.TranscriptionConfig(
                auto_chapters=True,
                punctuate=False,
            ),
        )
    # Check that the error message informs the user of the invalid parameter
    assert "punctuate" in str(error)

    # Check that the error was raised before any requests were made
    assert len(httpx_mock.get_requests()) == 0

    # Inform httpx_mock that it's okay we didn't make any requests
    httpx_mock.reset(False)


def test_auto_chapters_disabled_by_default(httpx_mock: HTTPXMock):
    """
    Tests that excluding `auto_chapters` from the `TranscriptionConfig` will
    result in the default behavior of it being excluded from the request body
    """
    request_body, transcript = __submit_mock_request(
        httpx_mock,
        mock_response=factories.generate_dict_factory(
            factories.TranscriptCompletedResponseFactory
        )(),
        config=aai.TranscriptionConfig(),
    )
    assert request_body.get("auto_chapters") is None
    assert transcript.chapters is None


def test_auto_chapters_enabled(httpx_mock: HTTPXMock):
    """
    Tests that including `auto_chapters=True` in the `TranscriptionConfig`
    will result in `auto_chapters=True` in the request body, and that the
    response is properly parsed into a `Transcript` object
    """
    mock_response = factories.generate_dict_factory(AutoChaptersResponseFactory)()
    request_body, transcript = __submit_mock_request(
        httpx_mock,
        mock_response=mock_response,
        config=aai.TranscriptionConfig(auto_chapters=True),
    )

    # Check that request body was properly defined
    assert request_body.get("auto_chapters") == True

    # Check that transcript was properly parsed from JSON response
    assert transcript.error is None
    assert transcript.chapters is not None
    assert len(transcript.chapters) > 0
    assert len(transcript.chapters) == len(mock_response["chapters"])

    for response_chapter, transcript_chapter in zip(
        mock_response["chapters"], transcript.chapters
    ):
        assert transcript_chapter.summary == response_chapter["summary"]
        assert transcript_chapter.headline == response_chapter["headline"]
        assert transcript_chapter.gist == response_chapter["gist"]
        assert transcript_chapter.start == response_chapter["start"]
        assert transcript_chapter.end == response_chapter["end"]



import copy
import os
from unittest.mock import mock_open, patch

import httpx
import pytest
from pytest_httpx import HTTPXMock

import assemblyai as aai
from tests.unit import factories

aai.settings.api_key = "test"


def test_submit_url_succeeds(httpx_mock: HTTPXMock):
    """
    Tests whether the submission of an URL works.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptProcessingResponseFactory
    )()

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=mock_transcript_response,
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()
    transcript = transcriber.submit("https://example.org/audio.wav")

    # ensure integrity
    assert transcript.id == mock_transcript_response["id"]

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 1


def test_submit_url_fails(httpx_mock: HTTPXMock):
    """
    Tests whether the submission of an URL fails.
    """

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="POST",
        json={"error": "something went wrong"},
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()
    transcript = transcriber.submit("https://example.org/audio.wav")

    # check whether the status is set to error
    assert transcript.audio_url == "https://example.org/audio.wav"
    assert transcript.status == aai.TranscriptStatus.error
    assert "something went wrong" in transcript.error

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 1


def test_submit_file_fails_due_api_error(httpx_mock: HTTPXMock):
    """
    Tests whether the submission of a file fails due to an API error.
    """

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/upload",
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="POST",
        json={"error": "something went wrong"},
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()

    # patch the reading of a local file
    with patch("builtins.open", mock_open(read_data=os.urandom(10))):
        transcript = transcriber.transcribe("audio.wav")

    # check whether the status is set to error
    assert transcript.audio_url == "audio.wav"
    assert transcript.status == aai.TranscriptStatus.error
    assert "something went wrong" in transcript.error

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 1


def test_submit_file_fails_due_file_not_found():
    """
    Tests whether the submission of a file fails due to a file not found error.
    """

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()

    # check whether the file not found error is raised
    with pytest.raises(FileNotFoundError):
        transcriber.submit("audio.wav")


def test_transcribe_url_succeeds(httpx_mock: HTTPXMock):
    """
    Tests whether the transcription of an URL works.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=mock_transcript_response,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_transcript_response['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_transcript_response,
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()
    transcript = transcriber.transcribe("https://example.org/audio.wav")

    # ensure integrity
    assert transcript.id == mock_transcript_response["id"]

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 2


def test_transcribe_file_succeeds(httpx_mock: HTTPXMock):
    """
    Tests whether the transcription of a local file works.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    # our local audio file that we want to transcribe
    local_file = os.urandom(10)

    # this is the url that we should receive when uploading the local file
    expected_upload_url = "https://example.org/audio.wav"
    mock_transcript_response["audio_url"] = expected_upload_url

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/upload",
        status_code=httpx.codes.OK,
        method="POST",
        json={"upload_url": expected_upload_url},
        match_content=local_file,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=mock_transcript_response,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_transcript_response['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_transcript_response,
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()

    # patch the reading of a local file
    with patch("builtins.open", mock_open(read_data=local_file)):
        transcript = transcriber.transcribe("audio.wav")

    # ensure integrity
    assert transcript.id == mock_transcript_response["id"]
    assert transcript.audio_url == expected_upload_url

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 3


def test_transcribe_group_urls_succeeds(httpx_mock: HTTPXMock):
    """
    Tests whether the transcription of multiple URLs work.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    # create different response mock objects
    response_1 = copy.deepcopy(mock_transcript_response)
    response_2 = copy.deepcopy(mock_transcript_response)

    expected_audio_urls = ["https://example.org/1.wav", "https://example.org/2.wav"]
    response_1["audio_url"] = expected_audio_urls[0]
    response_2["audio_url"] = expected_audio_urls[1]

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=response_1,
    )
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=response_2,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{response_1['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=response_1,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{response_2['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=response_2,
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()
    transcript_group = transcriber.transcribe_group(expected_audio_urls)

    # ensure integrity
    assert len(transcript_group.transcripts) == 2

    # check whether the audio urls match
    assert {t.audio_url for t in transcript_group} == set(expected_audio_urls)

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 4


def test_transcribe_group_urls_fails_during_upload(httpx_mock: HTTPXMock):
    """
    Tests the scenario of when a list of paths or URLs are being transcribed
    and one (or more) items of that list fail due a HTTP-error (e.g. Timeout, Internal Server Error, etc.)

    In this case we need to ensure that the application flow does not interrupt unexpectedly and the user
    is able to backtrace the reason of those failed uploads.
    """

    # create a mock response of a completed transcript
    succeeds_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    expect_succeeded_audio_url = "https://example.org/succeeds.wav"
    succeeds_response["audio_url"] = expect_succeeded_audio_url

    expect_failed_audio_url = "https://example.org/fails.wav"

    # mock the specific endpoints

    # the first file fails (see `.transcripe_group(...)` below)
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="POST",
        json={"error": "Aww, Snap!"},
    )

    # the second one succeeds (see `.transcribe_group(...) below`)
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=succeeds_response,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{succeeds_response['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=succeeds_response,
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()
    transcript_group = transcriber.transcribe_group(
        [
            expect_failed_audio_url,
            expect_succeeded_audio_url,
        ],
    )

    # ensure integrity
    assert len(transcript_group.transcripts) == 2

    # check whether the audio urls match
    audio_urls = {t.audio_url for t in transcript_group.transcripts}
    assert audio_urls == {expect_failed_audio_url, expect_succeeded_audio_url}

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 4


def test_transcribe_async_url_succeeds(httpx_mock: HTTPXMock):
    # create a mock response of a processing transcript
    mock_processing_response = factories.generate_dict_factory(
        factories.TranscriptProcessingResponseFactory
    )()

    # create a mock response of a completed transcript
    mock_completed_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()
    mock_completed_response["id"] = mock_processing_response["id"]

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=mock_processing_response,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_processing_response['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_processing_response,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_processing_response['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_completed_response,
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()
    transcript_future = transcriber.transcribe_async(
        "https://example.org/audio.wav",
    )

    # `transcribe` is being called with a callback the operation works asynchronously
    transcript = transcript_future.result()

    # ensure integrity
    assert transcript.id == mock_completed_response["id"]
    assert transcript.status == aai.TranscriptStatus.completed

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 3


def test_transcribe_async_url_fails(httpx_mock: HTTPXMock):
    # create a mock response of a processing transcript
    mock_processing_transcript = factories.generate_dict_factory(
        factories.TranscriptProcessingResponseFactory
    )()

    # create a mock response of a error transcript
    mock_error_transcript = factories.generate_dict_factory(
        factories.TranscriptErrorResponseFactory
    )()

    mock_error_transcript["id"] = mock_processing_transcript["id"]

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=mock_processing_transcript,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_processing_transcript['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_processing_transcript,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_error_transcript['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_error_transcript,
    )

    # mimic the usage of the SDK
    transcriber = aai.Transcriber()
    transcript_future = transcriber.transcribe_async(
        "https://example.org/audio.wav",
    )

    # `transcribe` is being called with a callback the operation works asynchronously
    transcript = transcript_future.result()

    # ensure integrity
    assert transcript.id == mock_error_transcript["id"]
    assert transcript.status == aai.TranscriptStatus.error

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 3



import json
from typing import Any, Dict, Tuple

import factory
import httpx
from pytest_httpx import HTTPXMock

import assemblyai as aai
from tests.unit import factories

aai.settings.api_key = "test"


class EntityFactory(factory.Factory):
    class Meta:
        model = aai.types.Entity

    entity_type = factory.Faker("enum", enum_cls=aai.types.EntityType)
    text = factory.Faker("sentence")
    start = factory.Faker("pyint")
    end = factory.Faker("pyint")


class EntityDetectionResponseFactory(factories.TranscriptCompletedResponseFactory):
    entities = factory.List([factory.SubFactory(EntityFactory)])


def __submit_mock_request(
    httpx_mock: HTTPXMock,
    mock_response: Dict[str, Any],
    config: aai.TranscriptionConfig,
) -> Tuple[Dict[str, Any], aai.Transcript]:
    """
    Helper function to abstract mock transcriber calls with given `TranscriptionConfig`,
    and perform some common assertions.
    """

    mock_transcript_id = mock_response.get("id", "mock_id")

    # Mock initial submission response (transcript is processing)
    mock_processing_response = factories.generate_dict_factory(
        factories.TranscriptProcessingResponseFactory
    )()

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json={
            **mock_processing_response,
            "id": mock_transcript_id,  # inject ID from main mock response
        },
    )

    # Mock polling-for-completeness response, with completed transcript
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_transcript_id}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_response,
    )

    # == Make API request via SDK ==
    transcript = aai.Transcriber().transcribe(
        data="https://example.org/audio.wav",
        config=config,
    )

    # Check that submission and polling requests were made
    assert len(httpx_mock.get_requests()) == 2

    # Extract body of initial submission request
    request = httpx_mock.get_requests()[0]
    request_body = json.loads(request.content.decode())

    return request_body, transcript


def test_entity_detection_disabled_by_default(httpx_mock: HTTPXMock):
    """
    Tests that excluding `entity_detection` from the `TranscriptionConfig` will
    result in the default behavior of it being excluded from the request body
    """
    request_body, transcript = __submit_mock_request(
        httpx_mock,
        mock_response=factories.generate_dict_factory(
            factories.TranscriptCompletedResponseFactory
        )(),
        config=aai.TranscriptionConfig(),
    )
    assert request_body.get("entity_detection") is None
    assert transcript.entities is None


def test_entity_detection_enabled(httpx_mock: HTTPXMock):
    """
    Tests that including `entity_detection=True` in the `TranscriptionConfig`
    will result in `entity_detection=True` in the request body, and that the
    response is properly parsed into a `Transcript` object
    """
    mock_response = factories.generate_dict_factory(EntityDetectionResponseFactory)()
    request_body, transcript = __submit_mock_request(
        httpx_mock,
        mock_response=mock_response,
        config=aai.TranscriptionConfig(entity_detection=True),
    )

    # Check that request body was properly defined
    assert request_body.get("entity_detection") == True

    # Check that transcript was properly parsed from JSON response
    assert transcript.error is None
    assert transcript.entities is not None
    assert len(transcript.entities) > 0
    assert len(transcript.entities) == len(mock_response["entities"])

    for entity in transcript.entities:
        assert len(entity.text.strip()) > 0



from typing import Any, Dict, List, Optional, Union

from . import api
from . import client as _client
from . import types


class _LemurImpl:
    def __init__(
        self,
        *,
        client: _client.Client,
        transcript_ids: List[str],
    ) -> None:
        self._client = client
        self._transcript_ids = transcript_ids

    def question(
        self,
        questions: List[types.LemurQuestion],
        model: types.LemurModel,
        timeout: Optional[float],
    ) -> List[types.LemurQuestionResult]:
        response = api.lemur_question(
            client=self._client.http_client,
            request=types.LemurQuestionRequest(
                transcript_ids=self._transcript_ids,
                questions=questions,
                model=model,
            ),
            http_timeout=timeout,
        )

        return response.response

    def summarize(
        self,
        context: Optional[Union[str, Dict[str, Any]]],
        answer_format: Optional[str],
        timeout: Optional[float],
    ) -> str:
        response = api.lemur_summarize(
            client=self._client.http_client,
            request=types.LemurSummaryRequest(
                transcript_ids=self._transcript_ids,
                context=context,
                answer_format=answer_format,
            ),
            http_timeout=timeout,
        )

        return response.response

    def ask_coach(
        self,
        context: Union[str, Dict[str, Any]],
        timeout: Optional[float],
    ) -> str:
        response = api.lemur_coach(
            client=self._client.http_client,
            request=types.LemurCoachRequest(
                transcript_ids=self._transcript_ids,
                context=context,
            ),
            http_timeout=timeout,
        )

        return response.response


class Lemur:
    def __init__(
        self,
        transcript_ids: List[str],
        client: Optional[_client.Client] = None,
    ) -> None:
        self._client = client or _client.Client.get_default()

        self._impl = _LemurImpl(
            client=self._client,
            transcript_ids=transcript_ids,
        )

    def question(
        self,
        questions: Union[types.LemurQuestion, List[types.LemurQuestion]],
        model: types.LemurModel = types.LemurModel.default,
        timeout: Optional[float] = None,
    ) -> Union[types.LemurQuestionResult, List[types.LemurQuestionResult]]:
        """
        Ask questions about transcripts powered by AssemblyAI's LeMUR model.

        Args:
            questions: One or a list of questions to ask.
            model: The LeMUR model to use for the question(s).
            timeout: The timeout in seconds to wait for the answer(s).

        Returns: One or a list of answer objects.
        """

        if not isinstance(questions, list):
            questions = [questions]

        answer = self._impl.question(
            questions=questions,
            model=model,
            timeout=timeout,
        )

        return answer[0] if len(answer) == 1 else answer

    def summarize(
        self,
        context: Optional[Union[str, Dict[str, Any]]] = None,
        answer_format: Optional[str] = None,
        timeout: Optional[float] = None,
    ) -> str:
        """
        Summarize the given transcript with the power of AssemblyAI's Lemur model.

        Args:
            context: An optional context on the transcript.
            answer_format: The format on how the summary shall be summarized.
            timeout: The timeout in seconds to wait for the summary.

        Returns: The summary as a string.
        """

        return self._impl.summarize(
            context=context,
            answer_format=answer_format,
            timeout=timeout,
        )

    def ask_coach(
        self,
        context: Union[str, Dict[str, Any]],
        timeout: Optional[float] = None,
    ) -> str:
        """
        Ask the AI coach a question on the transcript powered by AssemblyAI's Lemur model.

        Args:
            context: The context and/or question to ask the coach.
            timeout: The timeout in seconds to wait for the coach.

        Returns: The feedback of the AI's coach as a string.
        """

        return self._impl.ask_coach(
            context=context,
            timeout=timeout,
        )



import json
from typing import Any, Dict, Tuple

import factory
import httpx
import pytest
from pytest_httpx import HTTPXMock

import assemblyai as aai
from tests.unit import factories

aai.settings.api_key = "test"


def __submit_mock_request(
    httpx_mock: HTTPXMock,
    mock_response: Dict[str, Any],
    config: aai.TranscriptionConfig,
) -> Tuple[Dict[str, Any], aai.Transcript]:
    """
    Helper function to abstract mock transcriber calls with given `TranscriptionConfig`,
    and perform some common assertions.
    """

    mock_transcript_id = mock_response.get("id", "mock_id")

    # Mock initial submission response (transcript is processing)
    mock_processing_response = factories.generate_dict_factory(
        factories.TranscriptProcessingResponseFactory
    )()

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json={
            **mock_processing_response,
            "id": mock_transcript_id,  # inject ID from main mock response
        },
    )

    # Mock polling-for-completeness response, with completed transcript
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_transcript_id}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_response,
    )

    # == Make API request via SDK ==
    transcript = aai.Transcriber().transcribe(
        data="https://example.org/audio.wav",
        config=config,
    )

    # Check that submission and polling requests were made
    assert len(httpx_mock.get_requests()) == 2

    # Extract body of initial submission request
    request = httpx_mock.get_requests()[0]
    request_body = json.loads(request.content.decode())

    return request_body, transcript


def test_redact_pii_disabled_by_default(httpx_mock: HTTPXMock):
    """
    Tests that excluding `redact_pii` from the `TranscriptionConfig` will
    result in the default behavior of it being excluded from the request body
    """
    request_body, transcript = __submit_mock_request(
        httpx_mock,
        mock_response=factories.generate_dict_factory(
            factories.TranscriptCompletedResponseFactory
        )(),
        config=aai.TranscriptionConfig(),
    )
    assert request_body.get("redact_pii") is None
    assert request_body.get("redact_pii_audio") is None
    assert request_body.get("redact_pii_policies") is None
    assert request_body.get("redact_pii_sub") is None


def test_redact_pii_enabled(httpx_mock: HTTPXMock):
    """
    Tests that enabling `redact_pii`, along with the required `redact_pii_policies`
    parameter will result in the request body containing those fields
    """
    policies = [
        aai.types.PIIRedactionPolicy.date,
        aai.types.PIIRedactionPolicy.phone_number,
    ]

    request_body, _ = __submit_mock_request(
        httpx_mock,
        mock_response=factories.generate_dict_factory(
            factories.TranscriptCompletedResponseFactory
        )(),
        config=aai.TranscriptionConfig(
            redact_pii=True,
            redact_pii_policies=policies,
        ),
    )

    assert request_body.get("redact_pii") is True
    assert request_body.get("redact_pii_policies") == policies


def test_redact_pii_enabled_with_optional_params(httpx_mock: HTTPXMock):
    """
    Tests that enabling `redact_pii`, along with the other optional parameters
    relevant to PII redaction, will result in the request body containing
    those fields
    """
    policies = [
        aai.types.PIIRedactionPolicy.date,
        aai.types.PIIRedactionPolicy.phone_number,
    ]
    sub_type = aai.types.PIISubstitutionPolicy.entity_name

    request_body, _ = __submit_mock_request(
        httpx_mock,
        mock_response=factories.generate_dict_factory(
            factories.TranscriptCompletedResponseFactory
        )(),
        config=aai.TranscriptionConfig(
            redact_pii=True,
            redact_pii_audio=True,
            redact_pii_policies=policies,
            redact_pii_sub=sub_type,
        ),
    )

    assert request_body.get("redact_pii") is True
    assert request_body.get("redact_pii_audio") is True
    assert request_body.get("redact_pii_policies") == policies
    assert request_body.get("redact_pii_sub") == sub_type


def test_redact_pii_fails_without_policies(httpx_mock: HTTPXMock):
    """
    Tests that enabling `redact_pii` without specifying any policies
    will result in an exception being raised before the API call is made
    """
    with pytest.raises(ValueError) as error:
        __submit_mock_request(
            httpx_mock,
            mock_response={},
            config=aai.TranscriptionConfig(
                redact_pii=True,
                # No policies!
            ),
        )

    assert "policy" in str(error)

    # Check that the error was raised before any requests were made
    assert len(httpx_mock.get_requests()) == 0

    # Inform httpx_mock that it's okay we didn't make any requests
    httpx_mock.reset(False)


def test_redact_pii_params_excluded_when_disabled(httpx_mock: HTTPXMock):
    """
    Tests that additional PII redaction parameters are excluded from the submission
    request body if `redact_pii` itself is not enabled.
    """
    request_body, _ = __submit_mock_request(
        httpx_mock,
        mock_response=factories.generate_dict_factory(
            factories.TranscriptCompletedResponseFactory
        )(),
        config=aai.TranscriptionConfig(
            redact_pii=False,
            redact_pii_audio=True,
            redact_pii_policies=[aai.types.PIIRedactionPolicy.date],
            redact_pii_sub=aai.types.PIISubstitutionPolicy.entity_name,
        ),
    )

    assert request_body.get("redact_pii") is None
    assert request_body.get("redact_pii_audio") is None
    assert request_body.get("redact_pii_policies") is None
    assert request_body.get("redact_pii_sub") is None



import json
from typing import Any, Dict, Tuple

import factory
import httpx
from pytest_httpx import HTTPXMock

import assemblyai as aai
from tests.unit import factories

aai.settings.api_key = "test"


class AutohighlightResultFactory(factory.Factory):
    class Meta:
        model = aai.types.AutohighlightResult

    count = factory.Faker("pyint")
    rank = factory.Faker("pyfloat")
    text = factory.Faker("sentence")
    timestamps = factory.List([factory.SubFactory(factories.TimestampFactory)])


class AutohighlightResponseFactory(factory.Factory):
    class Meta:
        model = aai.types.AutohighlightResponse

    status = aai.types.StatusResult.success
    results = factory.List([factory.SubFactory(AutohighlightResultFactory)])


class AutohighlightTranscriptResponseFactory(
    factories.TranscriptCompletedResponseFactory
):
    auto_highlights_result = factory.SubFactory(AutohighlightResponseFactory)


def __submit_mock_request(
    httpx_mock: HTTPXMock,
    mock_response: Dict[str, Any],
    config: aai.TranscriptionConfig,
) -> Tuple[Dict[str, Any], aai.Transcript]:
    """
    Helper function to abstract mock transcriber calls with given `TranscriptionConfig`,
    and perform some common assertions.
    """

    mock_transcript_id = mock_response.get("id", "mock_id")

    # Mock initial submission response (transcript is processing)
    mock_processing_response = factories.generate_dict_factory(
        factories.TranscriptProcessingResponseFactory
    )()

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json={
            **mock_processing_response,
            "id": mock_transcript_id,  # inject ID from main mock response
        },
    )

    # Mock polling-for-completeness response, with completed transcript
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_transcript_id}",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_response,
    )

    # == Make API request via SDK ==
    transcript = aai.Transcriber().transcribe(
        data="https://example.org/audio.wav",
        config=config,
    )

    # Check that submission and polling requests were made
    assert len(httpx_mock.get_requests()) == 2

    # Extract body of initial submission request
    request = httpx_mock.get_requests()[0]
    request_body = json.loads(request.content.decode())

    return request_body, transcript


def test_auto_highlights_disabled_by_default(httpx_mock: HTTPXMock):
    """
    Tests that excluding `auto_highlights` from the `TranscriptionConfig` will
    result in the default behavior of it being excluded from the request body
    """
    request_body, transcript = __submit_mock_request(
        httpx_mock,
        mock_response=factories.generate_dict_factory(
            factories.TranscriptCompletedResponseFactory
        )(),
        config=aai.TranscriptionConfig(),
    )
    assert request_body.get("auto_highlights") is None
    assert transcript.auto_highlights is None


def test_auto_highlights_enabled(httpx_mock: HTTPXMock):
    """
    Tests that including `auto_highlights=True` in the `TranscriptionConfig`
    will result in `auto_highlights=True` in the request body, and that the
    response is properly parsed into a `Transcript` object
    """
    mock_response = factories.generate_dict_factory(
        AutohighlightTranscriptResponseFactory
    )()
    request_body, transcript = __submit_mock_request(
        httpx_mock,
        mock_response=mock_response,
        config=aai.TranscriptionConfig(auto_highlights=True),
    )

    # Check that request body was properly defined
    assert request_body.get("auto_highlights") == True

    # Check that transcript was properly parsed from JSON response
    assert transcript.error is None
    assert transcript.auto_highlights is not None
    assert (
        transcript.auto_highlights.status
        == mock_response["auto_highlights_result"]["status"]
    )

    assert transcript.auto_highlights.results is not None
    assert len(transcript.auto_highlights.results) > 0
    assert len(transcript.auto_highlights.results) == len(
        mock_response["auto_highlights_result"]["results"]
    )

    for response_result, transcript_result in zip(
        mock_response["auto_highlights_result"]["results"],
        transcript.auto_highlights.results,
    ):
        assert transcript_result.count == response_result["count"]
        assert transcript_result.rank == response_result["rank"]
        assert transcript_result.text == response_result["text"]

        for response_timestamp, transcript_timestamp in zip(
            response_result["timestamps"], transcript_result.timestamps
        ):
            assert transcript_timestamp.start == response_timestamp["start"]
            assert transcript_timestamp.end == response_timestamp["end"]



from typing import Any, Dict, List
from urllib.parse import urlencode

import httpx
import pytest
from faker import Faker
from pytest_httpx import HTTPXMock

import assemblyai as aai
from tests.unit import factories

aai.settings.api_key = "test"


def test_export_subtitles_succeeds(httpx_mock: HTTPXMock, faker: Faker):
    """
    Tests whether exporting subtitles succeed.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    expected_subtitles_srt = faker.text()
    expected_subtitles_vtt = faker.text()

    transcript = aai.Transcript.from_response(
        client=aai.Client.get_default(),
        response=aai.types.TranscriptResponse(**mock_transcript_response),
    )

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/srt",
        status_code=httpx.codes.OK,
        method="GET",
        text=expected_subtitles_srt,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/vtt",
        status_code=httpx.codes.OK,
        method="GET",
        text=expected_subtitles_vtt,
    )

    srt_subtitles = transcript.export_subtitles_srt()
    vtt_subtitles = transcript.export_subtitles_vtt()

    assert srt_subtitles == expected_subtitles_srt
    assert vtt_subtitles == expected_subtitles_vtt

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 2


def test_export_subtitles_fails(httpx_mock: HTTPXMock):
    """
    Tests whether exporting subtitles fails.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    transcript = aai.Transcript.from_response(
        client=aai.Client.get_default(),
        response=aai.types.TranscriptResponse(**mock_transcript_response),
    )

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/srt",
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="GET",
        json={"error": "something went wrong"},
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/vtt",
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="GET",
        json={"error": "something went wrong"},
    )

    with pytest.raises(aai.TranscriptError, match="something went wrong"):
        transcript.export_subtitles_srt()

    with pytest.raises(aai.TranscriptError, match="something went wrong"):
        transcript.export_subtitles_vtt()

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 2


def test_word_search_succeeds(httpx_mock: HTTPXMock):
    """
    Tests whether word search succeeds.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    transcript = aai.Transcript.from_response(
        client=aai.Client.get_default(),
        response=aai.types.TranscriptResponse(**mock_transcript_response),
    )

    # create a mock response for the word search
    mock_word_search_response = factories.generate_dict_factory(
        factories.WordSearchMatchResponseFactory
    )()

    search_words = {
        "words": ",".join(["test", "me"]),
    }
    # mock the specific endpoints
    url = httpx.URL(
        f"{aai.settings.base_url}/transcript/{transcript.id}/word-search?{urlencode(search_words)}",
    )

    httpx_mock.add_response(
        url=url,
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_word_search_response,
    )

    # mimic the SDK call
    matches = transcript.word_search(words=["test", "me"])

    # check integrity of the response

    for idx, word_search in enumerate(matches):
        assert isinstance(word_search, aai.WordSearchMatch)
        assert word_search.count == mock_word_search_response["matches"][idx]["count"]
        assert (
            word_search.timestamps
            == mock_word_search_response["matches"][idx]["timestamps"]
        )
        assert word_search.text == mock_word_search_response["matches"][idx]["text"]
        assert (
            word_search.indexes == mock_word_search_response["matches"][idx]["indexes"]
        )

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 1


def test_word_search_fails(httpx_mock: HTTPXMock):
    """
    Tests whether word search fails.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    transcript = aai.Transcript.from_response(
        client=aai.Client.get_default(),
        response=aai.types.TranscriptResponse(**mock_transcript_response),
    )

    # mock the specific endpoints
    url = httpx.URL(
        f"{aai.settings.base_url}/transcript/{transcript.id}/word-search?words=test",
    )

    httpx_mock.add_response(
        url=url,
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="GET",
        json={"error": "something went wrong"},
    )

    with pytest.raises(aai.TranscriptError, match="something went wrong"):
        transcript.word_search(words=["test"])

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 1


def test_get_sentences_and_paragraphs_succeeds(httpx_mock: HTTPXMock):
    """
    Tests whether getting sentences and paragraphs succeeds.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    # create a mock response for the sentences
    mock_sentences_response = factories.generate_dict_factory(
        factories.SentencesResponseFactory
    )()

    # create a mock response for the paragraphs
    mock_paragraphs_response = factories.generate_dict_factory(
        factories.ParagraphsResponseFactory
    )()

    transcript = aai.Transcript.from_response(
        client=aai.Client.get_default(),
        response=aai.types.TranscriptResponse(**mock_transcript_response),
    )

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/sentences",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_sentences_response,
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/paragraphs",
        status_code=httpx.codes.OK,
        method="GET",
        json=mock_paragraphs_response,
    )

    # mimic the SDK call
    sentences = transcript.get_sentences()
    paragraphs = transcript.get_paragraphs()

    # check integrity of the response
    def compare_words(lhs: List[aai.Word], rhs: List[Dict[str, Any]]) -> bool:
        """
        Compares the list of Word objects with the list of dicts.

        Args:
            lhs: The list of Word objects.
            rhs: The list of dicts.

        Returns:
            True if the lists are equal, False otherwise.
        """
        for idx, word in enumerate(lhs):
            if word.text != rhs[idx]["text"]:
                return False
            if word.start != rhs[idx]["start"]:
                return False
            if word.end != rhs[idx]["end"]:
                return False
        return True

    for idx, sentence in enumerate(sentences):
        assert isinstance(sentence, aai.Sentence)
        assert compare_words(
            sentence.words, mock_sentences_response["sentences"][idx]["words"]
        )

    for idx, paragraph in enumerate(paragraphs):
        assert isinstance(paragraph, aai.Paragraph)
        assert compare_words(
            paragraph.words, mock_paragraphs_response["paragraphs"][idx]["words"]
        )

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 2


def test_get_sentences_and_paragraphs_fails(httpx_mock: HTTPXMock):
    """
    Tests whether getting sentences and paragraphs fails.
    """

    # create a mock response of a completed transcript
    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    transcript = aai.Transcript.from_response(
        client=aai.Client.get_default(),
        response=aai.types.TranscriptResponse(**mock_transcript_response),
    )

    # mock the specific endpoints
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/sentences",
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="GET",
        json={"error": "something went wrong"},
    )

    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{transcript.id}/paragraphs",
        status_code=httpx.codes.INTERNAL_SERVER_ERROR,
        method="GET",
        json={"error": "something went wrong"},
    )

    # mimic the SDK call
    with pytest.raises(aai.TranscriptError, match="something went wrong"):
        transcript.get_sentences()
    with pytest.raises(aai.TranscriptError, match="something went wrong"):
        transcript.get_paragraphs()

    # check whether we mocked everything
    assert len(httpx_mock.get_requests()) == 2



from pathlib import Path

from setuptools import find_packages, setup

long_description = (Path(__file__).parent / "README.md").read_text()


setup(
    name="assemblyai",
    version="0.9.0",
    description="AssemblyAI Python SDK",
    author="AssemblyAI",
    author_email="engineering.sdk@assemblyai.com",
    packages=find_packages(),
    install_requires=[
        "httpx>=0.19.0",
        "pydantic>=1.7.0",
        "typing-extensions",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AssemblyAI/assemblyai-python-sdk",
    license="MIT License",
    license_files=["LICENSE"],
    python_requires=">=3.8",
    project_urls={
        "Code": "https://github.com/AssemblyAI/assemblyai-python-sdk",
        "Issues": "https://github.com/AssemblyAI/assemblyai-python-sdk/issues",
        "Documentation": "https://github.com/AssemblyAI/assemblyai-python-sdk/blob/master/README.md",
        "API Documentation": "https://www.assemblyai.com/docs/",
        "Website": "https://assemblyai.com/",
    },
)



import json
from typing import Any, Dict

import httpx
import pytest
from pytest_httpx import HTTPXMock

import assemblyai as aai
from tests.unit import factories

aai.settings.api_key = "test"


def __submit_request(httpx_mock: HTTPXMock, **params) -> Dict[str, Any]:
    """
    Helper function to abstract calling transcriber with given parameters,
    and perform some common assertions.

    Returns the body (dictionary) of the initial submission request.
    """
    summary = "example summary"

    mock_transcript_response = factories.generate_dict_factory(
        factories.TranscriptCompletedResponseFactory
    )()

    # Mock initial submission response
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript",
        status_code=httpx.codes.OK,
        method="POST",
        json=mock_transcript_response,
    )

    # Mock polling-for-completeness response, with mock summary result
    httpx_mock.add_response(
        url=f"{aai.settings.base_url}/transcript/{mock_transcript_response['id']}",
        status_code=httpx.codes.OK,
        method="GET",
        json={**mock_transcript_response, "summary": summary},
    )

    # == Make API request via SDK ==
    transcript = aai.Transcriber().transcribe(
        data="https://example.org/audio.wav",
        config=aai.TranscriptionConfig(
            **params,
        ),
    )

    # Check that submission and polling requests were made
    assert len(httpx_mock.get_requests()) == 2

    # Check that summary field from response was traced back through SDK classes
    assert transcript.summary == summary

    # Extract and return body of initial submission request
    request = httpx_mock.get_requests()[0]
    return json.loads(request.content.decode())


@pytest.mark.parametrize("required_field", ["punctuate", "format_text"])
def test_summarization_fails_without_required_field(
    httpx_mock: HTTPXMock, required_field: str
):
    """
    Tests whether the SDK raises an error before making a request
    if `summarization` is enabled and the given required field is disabled
    """
    with pytest.raises(ValueError) as error:
        __submit_request(httpx_mock, summarization=True, **{required_field: False})

    # Check that the error message informs the user of the invalid parameter
    assert required_field in str(error)

    # Check that the error was raised before any requests were made
    assert len(httpx_mock.get_requests()) == 0

    # Inform httpx_mock that it's okay we didn't make any requests
    httpx_mock.reset(False)


def test_summarization_disabled_by_default(httpx_mock: HTTPXMock):
    """
    Tests that excluding `summarization` from the `TranscriptionConfig` will
    result in the default behavior of it being excluded from the request body
    """
    request_body = __submit_request(httpx_mock)
    assert request_body.get("summarization") is None


def test_default_summarization_params(httpx_mock: HTTPXMock):
    """
    Tests that including `summarization=True` in the `TranscriptionConfig`
    will result in `summarization=True` in the request body.
    """
    request_body = __submit_request(httpx_mock, summarization=True)
    assert request_body.get("summarization") == True


def test_summarization_with_params(httpx_mock: HTTPXMock):
    """
    Tests that including additional summarization parameters along with
    `summarization=True` in the `TranscriptionConfig` will result in all
    parameters being included in the request as well.
    """

    summary_model = aai.SummarizationModel.conversational
    summary_type = aai.SummarizationType.bullets

    request_body = __submit_request(
        httpx_mock,
        summarization=True,
        summary_model=summary_model,
        summary_type=summary_type,
    )

    assert request_body.get("summarization") == True
    assert request_body.get("summary_model") == summary_model
    assert request_body.get("summary_type") == summary_type


def test_summarization_params_excluded_when_disabled(httpx_mock: HTTPXMock):
    """
    Tests that additional summarization parameters are excluded from the submission
    request body if `summarization` itself is not enabled.
    """
    request_body = __submit_request(
        httpx_mock,
        summarization=False,
        summary_model=aai.SummarizationModel.conversational,
        summary_type=aai.SummarizationType.bullets,
    )

    assert request_body.get("summarization") is None
    assert request_body.get("summary_model") is None
    assert request_body.get("summary_type") is None
